@@ -9,6 +9,7 @@ class Api(object):
 		self.url = url
 		self.headers = {'Authorization': 'PrivateToken ' + private_token}
 		mimetypes.add_type('text/plain', '.log')
+		mimetypes.add_type('application/octet-stream', '')
 
 	# AutoTests
 	def create_autotest(self, json):
@@ -60,12 +61,12 @@ class Api(object):
 		else:
 			print(f"Set results error: {response.json()['error']['key']}")
 
-	def testrun_activity(self, testrun_id, to_do):
-		response = requests.post(f'{self.url}/api/v2/testRuns/{testrun_id}/{to_do}', headers=self.headers)
+	def testrun_activity(self, testrun_id, action):
+		response = requests.post(f'{self.url}/api/v2/testRuns/{testrun_id}/{action}', headers=self.headers)
 		if response.status_code == 204:
-			print(f'TestRun {to_do} passed!')
+			print(f'TestRun {action} passed!')
 		else:
-			print(f"TestRun {to_do} error: {response.json()['error']['key']}")
+			print(f"TestRun {action} error: {response.json()['error']['key']}")
 
 	def link_attachment(self, test_result_id, file):
 		response = requests.post(f'{self.url}/api/v2/testResults/{test_result_id}/attachments', headers=self.headers, files={'file': (os.path.basename(file.name), file, mimetypes.guess_type(file.name)[0])})
