@@ -121,57 +121,9 @@ class TestITListener(object):
             data['steps'] = item.test_steps if hasattr(item, 'test_steps') else []
             data['stepResults'] = item.test_results_steps if hasattr(item, 'test_results_steps') else []
 
-            if hasattr(item, 'test_session_setup_steps'):
-                tree_setup_steps['session'] = item.test_session_setup_steps
-                tree_setup_steps['session_results'] = item.test_session_setup_results_steps
-            elif 'session' not in tree_setup_steps:
-                tree_setup_steps['session'] = []
-                tree_setup_steps['session_results'] = []
-            if hasattr(item, 'test_module_setup_steps'):
-                tree_setup_steps['module'] = item.test_module_setup_steps
-                tree_setup_steps['module_results'] = item.test_module_setup_results_steps
-            elif 'module' not in tree_setup_steps:
-                tree_setup_steps['module'] = []
-                tree_setup_steps['module_results'] = []
-            if hasattr(item, 'test_class_setup_steps'):
-                tree_setup_steps['class'] = item.test_class_setup_steps
-                tree_setup_steps['class_results'] = item.test_class_setup_results_steps
-            elif 'class' not in tree_setup_steps:
-                tree_setup_steps['class'] = []
-                tree_setup_steps['class_results'] = []
-            if hasattr(item, 'test_method_setup_steps'):
-                tree_setup_steps['method'] = item.test_method_setup_steps
-                tree_setup_steps['method_results'] = item.test_method_setup_results_steps
-            elif 'method' not in tree_setup_steps:
-                tree_setup_steps['method'] = []
-                tree_setup_steps['method_results'] = []
             data['setUp'] = tree_setup_steps['session'] + tree_setup_steps['module'] + tree_setup_steps['class'] + tree_setup_steps['method']
             data['setUpResults'] = tree_setup_steps['session_results'] + tree_setup_steps['module_results'] + tree_setup_steps['class_results'] + tree_setup_steps['method_results']
 
-            if hasattr(item, 'test_session_teardown_steps'):
-                tree_teardown_steps['session'] = item.test_session_teardown_steps
-                tree_teardown_steps['session_results'] = item.test_session_teardown_results_steps
-            elif 'session' not in tree_teardown_steps:
-                tree_teardown_steps['session'] = []
-                tree_teardown_steps['session_results'] = []
-            if hasattr(item, 'test_module_teardown_steps'):
-                tree_teardown_steps['module'] = item.test_module_teardown_steps
-                tree_teardown_steps['module_results'] = item.test_module_teardown_results_steps
-            elif 'module' not in tree_teardown_steps:
-                tree_teardown_steps['module'] = []
-                tree_teardown_steps['module_results'] = []
-            if hasattr(item, 'test_class_teardown_steps'):
-                tree_teardown_steps['class'] = item.test_class_teardown_steps
-                tree_teardown_steps['class_results'] = item.test_class_teardown_results_steps
-            elif 'class' not in tree_teardown_steps:
-                tree_teardown_steps['class'] = []
-                tree_teardown_steps['class_results'] = []
-            if hasattr(item, 'test_method_teardown_steps'):
-                tree_teardown_steps['method'] = item.test_method_teardown_steps
-                tree_teardown_steps['method_results'] = item.test_method_teardown_results_steps
-            elif 'method' not in tree_teardown_steps:
-                tree_teardown_steps['method'] = []
-                tree_teardown_steps['method_results'] = []
             data['tearDown'] = tree_teardown_steps['method'] + tree_teardown_steps['class'] + tree_teardown_steps['module'] + tree_teardown_steps['session']
             data['tearDownResults'] = tree_teardown_steps['method_results'] + tree_teardown_steps['class_results'] + tree_teardown_steps['module_results'] + tree_teardown_steps['session_results']
 
@@ -189,26 +141,15 @@ class TestITListener(object):
             data['links'] = []
             if hasattr(item.function, 'test_links'):
                 if parametrize_id:
-                    for ID in parametrize_id:
-                        mark = item.own_markers[ID]
-                        for link in item.function.test_links:
-                            data['links'].append({})
-                            if link['url'][link['url'].find('{') + 1:link['url'].rfind('}')] in item.own_markers[ID].args[0]:
-                                data['links'][-1]['url'] = link['url'].split('{')[0] + mark.args[1][index][mark.args[0].split(', ').index(link['url'][link['url'].find('{') + 1:link['url'].rfind('}')])] + link['url'].split('}')[1]
-                            else:
-                                data['links'][-1]['url'] = link['url']
-                            if link['title'] and link['title'][link['title'].find('{') + 1:link['title'].rfind('}')] in mark.args[0]:
-                                data['links'][-1]['title'] = link['title'].split('{')[0] + mark.args[1][index][mark.args[0].split(', ').index(link['title'][link['title'].find('{') + 1:link['title'].rfind('}')])] + link['title'].split('}')[1]
-                            else:
-                                data['links'][-1]['title'] = link['title']
-                            if link['type'] and link['type'][link['type'].find('{') + 1:link['type'].rfind('}')] in mark.args[0]:
-                                data['links'][-1]['type'] = link['type'].split('{')[0] + mark.args[1][index][mark.args[0].split(', ').index(link['type'][link['type'].find('{') + 1:link['type'].rfind('}')])] + link['type'].split('}')[1]
-                            else:
-                                data['links'][-1]['type'] = link['type']
-                            if link['description'] and link['description'][link['description'].find('{') + 1:link['description'].rfind('}')] in mark.args[0]:
-                                data['links'][-1]['description'] = link['description'].split('{')[0] + mark.args[1][index][mark.args[0].split(', ').index(link['description'][link['description'].find('{') + 1:link['description'].rfind('}')])] + link['description'].split('}')[1]
-                            else:
-                                data['links'][-1]['description'] = link['description']
+                    for link in item.function.test_links:
+                        data['links'].append({})
+                        data['links'][-1]['url'] = cls.attribute_collector_links(link, 'url', item.own_markers, parametrize_id, index)
+                        if link['title']:
+                            data['links'][-1]['title'] = cls.attribute_collector_links(link, 'title', item.own_markers, parametrize_id, index)
+                        if link['type']:
+                            data['links'][-1]['type'] = cls.attribute_collector_links(link, 'type', item.own_markers, parametrize_id, index)
+                        if link['description']:
+                            data['links'][-1]['description'] = cls.attribute_collector_links(link, 'description', item.own_markers, parametrize_id, index)
                 else:
                     data['links'] = item.function.test_links
 
@@ -219,7 +160,7 @@ class TestITListener(object):
             if hasattr(item.function, 'test_labels'):
                 if parametrize_id:
                     result, param_id = cls.mass_param_attribute_collector(item.function.test_labels[0], item.own_markers, parametrize_id, index)
-                    if param_id and item.function.test_labels[0][1:-1] in item.name[item.name.find('[') + 1:item.name.rfind(']')].split('-')[param_id]:
+                    if param_id is not None and item.function.test_labels[0][1:-1] in item.name[item.name.find('[') + 1:item.name.rfind(']')].split('-')[param_id]:
                         for label in result:
                             data['labels'].append({'name': label})
                     else:
@@ -232,7 +173,7 @@ class TestITListener(object):
             if hasattr(item.function, 'test_workItemsID'):
                 if parametrize_id:
                     result, param_id = cls.mass_param_attribute_collector(item.function.test_workItemsID[0], item.own_markers, parametrize_id, index)
-                    if param_id and item.function.test_workItemsID[0][1:-1] in item.name[item.name.find('[') + 1:item.name.rfind(']')].split('-')[param_id]:
+                    if param_id is not None and item.function.test_workItemsID[0][1:-1] in item.name[item.name.find('[') + 1:item.name.rfind(']')].split('-')[param_id]:
                         data['workItemsID'] = result
                     else:
                         data['workItemsID'] = [result]
@@ -265,8 +206,8 @@ class TestITListener(object):
                 data['failureReasonName'] = 'TestDefect'
                 data['traces'] = f'>\n{inspect.getsource(item.function)}\nE {item.originalname} must have @testit.displayName or documentation!\n{item.location[0]}:{item.location[1]}: Exception'
 
-            return data, tree_setup_steps, tree_teardown_steps, index + 1 if nextitem and item.originalname == nextitem.originalname else 0
-        return None, tree_setup_steps, tree_teardown_steps, 0
+            return data, index + 1 if nextitem and item.originalname == nextitem.originalname else 0
+        return None, 0
 
     @staticmethod
     def param_attribute_collector(attribute, marks, parametrize_id, index):
@@ -283,6 +224,24 @@ class TestITListener(object):
                 return marks[ID].args[1][index][param_id], param_id
         return attribute, None
 
+    @staticmethod
+    def attribute_collector_links(link, key, marks, parametrize_id, index):
+        for ID in parametrize_id:
+            if link[key][link[key].find('{') + 1:link[key].rfind('}')] in marks[ID].args[0]:
+                return link[key].split('{')[0] + marks[ID].args[1][index][marks[ID].args[0].split(', ').index(link[key][link[key].find('{') + 1:link[key].rfind('}')])] + link[key].split('}')[1]
+        return link[key]
+
+    @staticmethod
+    def form_tree_steps(item, tree_steps, stage):
+        for scope in ('session', 'module', 'class', 'method'):
+            if hasattr(item, f'test_{scope}_{stage}_steps'):
+                tree_steps[scope] = getattr(item, f'test_{scope}_{stage}_steps')
+                tree_steps[f'{scope}_results'] = getattr(item, f'test_{scope}_{stage}_results_steps')
+            elif scope not in tree_steps:
+                tree_steps[scope] = []
+                tree_steps[f'{scope}_results'] = []
+        return tree_steps
+
     @pytest.hookimpl
     def pytest_sessionfinish(self, session):
         index = 0
@@ -291,7 +250,15 @@ class TestITListener(object):
         tests_results_data = []
         data_attachments = []
         for item_id in range(len(session.items)):
-            data_item, tree_setup_steps, tree_teardown_steps, index = self.attribute_collector(session.items[item_id], session.items[item_id + 1] if item_id + 1 < len(session.items) else None, tree_setup_steps, tree_teardown_steps, index)
+            tree_setup_steps = self.form_tree_steps(session.items[item_id], tree_setup_steps, 'setup')
+            tree_teardown_steps = self.form_tree_steps(session.items[item_id], tree_teardown_steps, 'teardown')
+            data_item, index = self.attribute_collector(
+                                    session.items[item_id],
+                                    session.items[item_id + 1] if item_id + 1 < len(session.items) else None,
+                                    tree_setup_steps,
+                                    tree_teardown_steps,
+                                    index
+                                )
             if data_item:
                 autotest = self.requests.get_autotest(data_item['externalID'], self.parser.get('testit', 'projectID')).json()
                 if not autotest:
@@ -349,6 +316,7 @@ class TestITListener(object):
                                 autotest_id
                             )
                         )
+
                 for workitem_id in data_item['workItemsID']:
                     self.requests.link_autotest(autotest_id, workitem_id)
 
@@ -366,7 +334,8 @@ class TestITListener(object):
                         data_item['resultLinks'],
                         data_item['duration'],
                         data_item['failureReasonName'],
-                        data_item['message']
+                        data_item['message'],
+                        None
                     )
                 )
         if tests_results_data:
